//
//  collageAppDelegate.m
//  collage
//
//  Created by Shannon Appelcline on 9/16/08.
//  Copyright __MyCompanyName__ 2008. All rights reserved.
//

#import "collageViewController.h"

@implementation collageViewController

- (void)viewDidLoad {

	UIBarButtonItem *picButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAction target:self action:@selector(choosePic:)];
	UIBarButtonItem *camButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCamera target:self action:@selector(takePic:)];
	UIBarButtonItem *saveButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemSave target:self action:@selector(savePic:)];

	picButton.style = UIBarButtonItemStyleBordered;
	camButton.style = UIBarButtonItemStyleBordered;
	
	if ([UIImagePickerController isSourceTypeAvailable:UIImagePickerControllerSourceTypeCamera]) {
		origToolbar = [[NSArray alloc] initWithObjects:
					   picButton,camButton,saveButton,nil];
	} else if ([UIImagePickerController isSourceTypeAvailable:UIImagePickerControllerSourceTypePhotoLibrary]) {
		origToolbar = [[NSArray alloc] initWithObjects:
					   picButton,saveButton,nil];
	} else {
		exit(0);
	}

	[myTools setItems:origToolbar animated:NO];	
	
	[picButton release];
	[camButton release];
	
	[super viewDidLoad];
}

-(IBAction)choosePic:(id)sender {
		
	UIImagePickerController *myImagePicker = [[UIImagePickerController alloc] init];
	myImagePicker.delegate = self;
	myImagePicker.allowsEditing = YES;

	// Updated for ipad
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= 30200
	UIPopoverController *popover = [[UIPopoverController alloc] initWithContentViewController:myImagePicker];
	[popover presentPopoverFromBarButtonItem:sender permittedArrowDirections:UIPopoverArrowDirectionAny animated:YES];
#else
	[self presentModalViewController:myImagePicker animated:YES];	
#endif
	
}

-(IBAction)takePic:(id)sender {
	
	UIImagePickerController *myImagePicker = [[UIImagePickerController alloc] init];
	myImagePicker.sourceType = UIImagePickerControllerSourceTypeCamera;
	myImagePicker.delegate = self;
	myImagePicker.allowsEditing = NO;
	
	[self presentModalViewController:myImagePicker animated:YES];
}

- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingImage:(UIImage *)image editingInfo:(NSDictionary *)editingInfo {

	[self dismissModalViewControllerAnimated:YES];
	[picker release];

	float percentage = [self scaleImage:image] / 2;	
	startingSize = CGSizeMake(image.size.width*percentage,image.size.height*percentage);
	
	myImageView = [[tempImageView alloc] 
				   initWithFrame:CGRectMake(80,115,
											startingSize.width,
											startingSize.height)];
	myImageView.image = image;
	myImageView.userInteractionEnabled = YES;
	
	[self.view addSubview:myImageView];
	
	[myTools setItems:[NSArray arrayWithObject:
					   [[UIBarButtonItem alloc]
						initWithBarButtonSystemItem:UIBarButtonSystemItemDone
						target:self action:@selector(finishPic:)]] animated:YES];	

	// Update for ipad
	//mySlider = [[UISlider alloc] initWithFrame:CGRectMake(90,415,210,44)];
	//[self.view addSubview:mySlider];
	
	mySlider = [[UISlider alloc] initWithFrame:CGRectMake(90,0,210,44)];
	[myTools addSubview:mySlider];
	
	mySlider.value = .5;
	[mySlider addTarget:self action:@selector(rescalePic:) forControlEvents:UIControlEventValueChanged];
	
}

- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker {
	
	[self dismissModalViewControllerAnimated:YES];
	[picker release];
	
}

-(void)rescalePic:(id)sender {

	myImageView.frame = CGRectMake(myImageView.frame.origin.x,
								   myImageView.frame.origin.y,
								   startingSize.width * mySlider.value * 2,
								   startingSize.height * mySlider.value * 2);
	
}

-(void)finishPic:(id)sender {

	[(collageView *) self.view addPic:myImageView.image at:myImageView.frame];

	[myImageView removeFromSuperview];
	[myImageView release];
	[mySlider removeFromSuperview];
	[mySlider release];
	
	[myTools setItems:origToolbar animated:NO];		


}

-(float)scaleImage:(UIImage *)image {

	float toSize = 1.0;
	
	// Updated for ipad
	CGRect scRect = [[UIScreen mainScreen] applicationFrame];
	
	if (image.size.width * toSize > scRect.size.width) {
		toSize = scRect.size.width / image.size.width;
	}
	if (image.size.height * toSize > (scRect.size.height - 20)) {
		toSize = (scRect.size.height - 20) / image.size.height;
	}

	return toSize;
}

-(void)savePic:(id)sender {
	UIGraphicsBeginImageContext(self.view.bounds.size);
	[self.view.layer renderInContext:UIGraphicsGetCurrentContext()];
	myTools.hidden = YES;
	UIImage *collageImage = UIGraphicsGetImageFromCurrentImageContext();
	myTools.hidden = NO;
	UIGraphicsEndImageContext();
	
	UIImageWriteToSavedPhotosAlbum(collageImage,nil,nil,nil);
}

/*
 Implement loadView if you want to create a view hierarchy programmatically
- (void)loadView {
}
 */




- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	// Return YES for supported orientations
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= 30200
	return YES;
#else
	return (interfaceOrientation == UIInterfaceOrientationPortrait);	
#endif
}


- (void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning]; // Releases the view if it doesn't have a superview
	// Release anything that's not essential, such as cached data
}


- (void)dealloc {
	[myTools release];
	[super dealloc];
}

@end
