//
//  RootViewController.m
//  CDJournal
//
//  Created by Brandon Trebitowski on 11/2/09.
//  Copyright RightSprite 2009. All rights reserved.
//

#import "RootViewController.h"
#import "AddEntryViewController.h"
#import "CDJournalAppDelegate.h"
#import "Entry.h"

@implementation RootViewController

@synthesize managedObjectContext;
@synthesize entries;

- (void)viewDidLoad {
    [super viewDidLoad];

	CDJournalAppDelegate * delegate = (CDJournalAppDelegate *)[[UIApplication sharedApplication] delegate];
	self.managedObjectContext = [delegate managedObjectContext];
	[delegate release];
	
    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
    self.navigationItem.rightBarButtonItem = self.editButtonItem;
	self.navigationItem.leftBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd 
																						  target:self action:@selector(add:)];
	
	NSMutableArray * _entriesArray = [[NSMutableArray alloc] init];
	self.entries = _entriesArray;
	[_entriesArray release];
}

- (void) add:(id) sender {
	AddEntryViewController * addEntryViewController = [[AddEntryViewController alloc]
													   initWithNibName:@"AddEntryViewController"
													   bundle:[NSBundle mainBundle]];
	[addEntryViewController setManagedObjectContext:[self managedObjectContext]];
	[self presentModalViewController:addEntryViewController animated:YES];
	[addEntryViewController release];
}


- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
	// Fetch the results
	NSFetchRequest *request = [[NSFetchRequest alloc] init];
	NSEntityDescription *entity = [NSEntityDescription entityForName:@"Entry" inManagedObjectContext:managedObjectContext];
	[request setEntity:entity];
	
	// Sort the results
	NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"creationDate" ascending:NO];
	NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:sortDescriptor, nil];
	[request setSortDescriptors:sortDescriptors];
	[sortDescriptors release];
	[sortDescriptor release];
	
	// Execute the request
	NSError *error;
	NSMutableArray *mutableFetchResults = [[managedObjectContext executeFetchRequest:request error:&error] mutableCopy];
	if (mutableFetchResults == nil) {
		NSLog(@"Error fetching result %@",[error description]);
	}
	
	[self setEntries:mutableFetchResults];
	[mutableFetchResults release];
	[request release];
	[self.tableView reloadData];
}

/*
- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}
*/
/*
- (void)viewWillDisappear:(BOOL)animated {
	[super viewWillDisappear:animated];
}
*/
/*
- (void)viewDidDisappear:(BOOL)animated {
	[super viewDidDisappear:animated];
}
*/

/*
 // Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	// Return YES for supported orientations.
	return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
 */

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release anything that can be recreated in viewDidLoad or on demand.
	// e.g. self.myOutlet = nil;
}


#pragma mark Table view methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}


// Customize the number of rows in the table view.
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [self.entries count];
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
	static NSDateFormatter *dateFormatter = nil;
    if (dateFormatter == nil) {
        dateFormatter = [[NSDateFormatter alloc] init];
        [dateFormatter setTimeStyle:NSDateFormatterMediumStyle];
        [dateFormatter setDateStyle:NSDateFormatterMediumStyle];
    }
	
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier] autorelease];
    }
    
	// Configure the cell.
	Entry * entry = [self.entries objectAtIndex:indexPath.row];
	cell.textLabel.text = [entry title];
	cell.detailTextLabel.text = [dateFormatter stringFromDate:[entry creationDate]];
	cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    return cell;
}




// Override to support row selection in the table view.
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {

    AddEntryViewController * addEntryViewController = [[AddEntryViewController alloc] initWithNibName:@"AddEntryViewController" bundle:[NSBundle mainBundle]];
	[addEntryViewController setManagedObjectContext:self.managedObjectContext];
	[addEntryViewController setEntry:[entries objectAtIndex:indexPath.row]];
	[self.navigationController pushViewController:addEntryViewController animated:YES];
	[addEntryViewController release];
}



/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/



// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
 
        // Delete the managed object at the given index path.
        NSManagedObject *eventToDelete = [entries objectAtIndex:indexPath.row];
        [managedObjectContext deleteObject:eventToDelete];
 
        // Update the array and table view.
        [entries removeObjectAtIndex:indexPath.row];
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:YES];
 
        // Commit the change.
        NSError *error;
        if (![managedObjectContext save:&error]) {
            NSLog(@"Error deleted entry %@",[error description]);
        }
    }
}



/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
}
*/


/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/


- (void)dealloc {
    [super dealloc];
	[entries release];
	[managedObjectContext release];
}


@end

