//
//  SKDatabase.m
//  dbnav
//
//  Created by Shannon Appelcline on 9/11/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "SKDatabase.h"

@implementation SKDatabase

- (id)initWithFile:(NSString *)dbFile {
	
	self = [super init];
	
	NSString *paths = [[NSBundle mainBundle] resourcePath];
    NSString *path = [paths stringByAppendingPathComponent:dbFile];
	
	int result = sqlite3_open([path UTF8String], &dbh);
	NSAssert1(SQLITE_OK == result, NSLocalizedStringFromTable(@"Unable to open the sqlite database (%@).", @"Database", @""), [NSString stringWithUTF8String:sqlite3_errmsg(dbh)]);	
	
	return self;
	
}

- (void)close {
	
	if (dbh) {
		sqlite3_close(dbh);
	}
}

- (sqlite3 *)dbh {
	
	return dbh;
}

- (sqlite3_stmt *)prepare:(NSString *)sql {
	
	const char *utfsql = [sql UTF8String];
	
	sqlite3_stmt *statement;
	
	if (sqlite3_prepare([self dbh],utfsql,-1,&statement,NULL) == SQLITE_OK) {
		return statement;
	} else {
		return 0;
	}
}

- (id)lookupSingularSQL:(NSString *)sql forType:(NSString *)rettype {
	
	sqlite3_stmt *statement;
	id result;
	
	if (statement = [self prepare:sql]) {
		if (sqlite3_step(statement) == SQLITE_ROW) {		
			if ([rettype compare:@"text"] == NSOrderedSame) {
				result = [NSString stringWithUTF8String:(char *)sqlite3_column_text(statement,0)];
			} else if ([rettype compare:@"integer"] == NSOrderedSame) {
				result = (id)sqlite3_column_int(statement,0);
            }
		}
	}
	sqlite3_finalize(statement);
	return result;
	
}

@end
