//
//  GKTennisViewController.m
//  GKTennis
//
//  Created by Brandon Trebitowski on 10/14/09.
//  Copyright RightSprite 2009. All rights reserved.
//

#import "GKTennisViewController.h"

@implementation GKTennisViewController

@synthesize paddle_1;
@synthesize paddle_2;
@synthesize ball;
@synthesize score_1_label;
@synthesize score_2_label;
@synthesize game_label;

@synthesize gameState;
@synthesize peerStatus;

@synthesize gameSession;
@synthesize gamePeerId;

#pragma mark dealloc

- (void)viewDidUnload {
	self.paddle_1 = nil;
	self.paddle_1 = nil;
	self.ball	  = nil;
	self.score_1_label = nil;
	self.score_2_label = nil;
	
}

- (void)dealloc {
    [super dealloc];
}

#pragma mark initialization

- (void)viewDidLoad {
    [super viewDidLoad];
	
	self.gameState  = kStateStartGame;
	
	NSString *uid = [[UIDevice currentDevice] uniqueIdentifier];
	gameUniqueID = [uid hash];
	
	[self resetBall];
	
	CGRect scRect = [[UIScreen mainScreen] applicationFrame];
	
	gameStatus.paddlePosition[0].x = gameStatus.paddlePosition[1].x = scRect.size.width/2;
	
	[NSTimer scheduledTimerWithTimeInterval:0.01 target:self selector:@selector(gameLoop) userInfo:nil repeats:YES];
}

#pragma mark Touches Methods
-(void) touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
	[self touchesMoved:touches withEvent:event];
}

-(void) touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
	
	switch (self.gameState) {
		case kStateStartGame:
		{
			self.game_label.hidden = YES;
			[self showPicker];
		}
			break;
		case kStateMultiplayer:
		{
			UITouch * t = [[event allTouches] anyObject];
			CGPoint paddlePoint = [t locationInView:self.view];
			
			if(self.peerStatus == kServer) {
				paddle_1.center = CGPointMake(paddlePoint.x, paddle_1.center.y);
				gameStatus.paddlePosition[self.peerStatus].x = paddle_1.center.x;
			} else {
				paddle_2.center = CGPointMake(paddlePoint.x, paddle_2.center.y);
				gameStatus.paddlePosition[self.peerStatus].x = paddle_2.center.x;
			}
			
			[self sendNetworkPacket:gameSession packetID:NETWORK_MOVE_EVENT withData:&gameStatus ofLength:sizeof(gameStatus) reliable: NO];
		}
			break;
		case kStateGameOver:
			exit(0);
			break;
		default:
			break;
	}
}

#pragma mark game logic
-(void) gameLoop {
	CGRect scRect = [[UIScreen mainScreen] applicationFrame];
	
	switch (self.gameState) {
		case kStatePicker:
		case kStateStartGame:
			break;
		case kStateMultiplayerCointoss:
		{
			[self sendNetworkPacket:self.gameSession packetID:NETWORK_COINTOSS withData:&gameUniqueID ofLength:sizeof(int) reliable:YES];
			self.gameState = kStateMultiplayer; 
		}
			break;
		case kStateMultiplayer:
		{
			BOOL collision = NO;
			if(self.peerStatus == kServer) {
				// Detect wall collision
				CGPoint bottomRight = CGPointMake(ball.frame.origin.x + ball.frame.size.width, ball.frame.origin.y + ball.frame.size.height);
				
				if(gameStatus.ballPosition.y <= 0 ) {
					gameStatus.score[kClient]++;
					[self resetBall];
					return;
				}
				
				if(gameStatus.ballPosition.y >= scRect.size.height ) {
					gameStatus.score[kServer]++;
					[self resetBall];
					return;
				}
				
				if(ball.frame.origin.x <= 0 || bottomRight.x >= scRect.size.width) {
					gameStatus.ballVelocity.x *= -1;
					collision = YES;
				}
				
				if(collision)
					[self sendNetworkPacket:gameSession packetID:NETWORK_BALL_MOVE_EVENT withData:&gameStatus ofLength:sizeof(gameInfo) reliable:NO];
				
				// Detect paddle collision
				if((CGRectIntersectsRect(ball.frame, paddle_1.frame) ||
				   CGRectIntersectsRect(ball.frame, paddle_2.frame)) && !justCollided) {
					gameStatus.ballVelocity.y *= -1;
					collision = YES;
					justCollided = YES;
					[self performSelector:@selector(resetCollision) withObject:nil afterDelay:1.0];
				}
				
				paddle_2.center = CGPointMake(gameStatus.paddlePosition[1-self.peerStatus].x, 
											  paddle_2.center.y);
				
			} else {
				paddle_1.center = CGPointMake(gameStatus.paddlePosition[1-self.peerStatus].x
											  , paddle_1.center.y);
			}
			
			gameStatus.ballPosition.y = gameStatus.ballPosition.y + gameStatus.ballVelocity.y;
			gameStatus.ballPosition.x = gameStatus.ballPosition.x + gameStatus.ballVelocity.x;
			
			ball.center = CGPointMake(gameStatus.ballPosition.x,gameStatus.ballPosition.y );
			
			if(gameStatus.score[kServer] >= 5) {
				self.game_label.text = @"Player 1 wins!";
				self.game_label.hidden = NO;
				self.gameState = kStateGameOver;
				[self sendNetworkPacket:gameSession packetID:NETWORK_GAME_STATUS withData:&gameStatus ofLength:sizeof(gameInfo) reliable:YES];
				return;
			}
			
			if(gameStatus.score[kClient] >= 5) {
				self.game_label.text = @"Player 2 wins!";
				self.game_label.hidden = NO;
				self.gameState = kStateGameOver;
				[self sendNetworkPacket:gameSession packetID:NETWORK_GAME_STATUS withData:&gameStatus ofLength:sizeof(gameInfo) reliable:YES];
				return;
			}
		}
			break;
		default:
			break;
	}
}

- (void) resetCollision {
	justCollided = NO;
}

- (void) resetBall {	
	CGRect scRect = [[UIScreen mainScreen] applicationFrame];
	gameStatus.ballPosition.x = scRect.size.width/2;
	gameStatus.ballPosition.y = scRect.size.height/2;
	float isNegative = random() % 2;
	int direction = (isNegative < 1) ? -1 : 1;	
	gameStatus.ballVelocity.x = 4 * direction;
	gameStatus.ballVelocity.y = 4 * direction;	  
	score_1_label.text = [NSString stringWithFormat:@"%d",gameStatus.score[kServer]];
	score_2_label.text = [NSString stringWithFormat:@"%d",gameStatus.score[kClient]];
	if(gameSession)
		[self sendNetworkPacket:gameSession packetID:NETWORK_GAME_STATUS withData:&gameStatus ofLength:sizeof(gameInfo) reliable:YES];
}

#pragma mark GKPeerPicker stuff
-(void) showPicker {
	self.gameState = kStatePicker;
	GKPeerPickerController * picker = [[GKPeerPickerController alloc] init];
	picker.delegate = self;
	[picker show];
}

#pragma mark GKPeerPickerControllerDelegate methods
- (void)peerPickerControllerDidCancel:(GKPeerPickerController *)picker { 
    
	picker.delegate = nil;
    [picker autorelease]; 
	
	if(self.gameSession != nil)	{
		[self invalidateSession:self.gameSession];
		self.gameSession = nil;
	}
	 
	self.gameState = kStateStartGame;
	self.game_label.hidden = NO;
} 

- (GKSession *)peerPickerController:(GKPeerPickerController *)picker sessionForConnectionType:(GKPeerPickerConnectionType)type { 
	GKSession *session = [[GKSession alloc] initWithSessionID:@"GKTennis" displayName:nil sessionMode:GKSessionModePeer]; 
	return [session autorelease]; 
}

- (void)peerPickerController:(GKPeerPickerController *)picker didConnectPeer:(NSString *)peerID toSession:(GKSession *)session { 
	// Remember the current peer.
	self.gamePeerId = peerID;  // copy
	
	// Make sure we have a reference to the game session and it is set up
	self.gameSession = session; // retain
	self.gameSession.delegate = self; 
	[self.gameSession setDataReceiveHandler:self withContext:NULL];
	
	// Done with the Peer Picker so dismiss it.
	[picker dismiss];
	picker.delegate = nil;
	[picker autorelease];
	
	// Start Multiplayer game by entering a cointoss state to determine who is server/client.
	self.gameState = kStateMultiplayerCointoss;
} 

#pragma mark Session handling (borrowed from apple's GKTanks)
- (void)invalidateSession:(GKSession *)session {
	if(session != nil) {
		[session disconnectFromAllPeers]; 
		session.available = NO; 
		[session setDataReceiveHandler: nil withContext: NULL]; 
		session.delegate = nil; 
	}
}

#pragma mark send/receive methods
- (void)sendNetworkPacket:(GKSession *)session packetID:(int)packetID withData:(void *)data ofLength:(int)length reliable:(BOOL)howtosend {

	// the packet we'll send is resued
	static unsigned char networkPacket[1024];
	const unsigned int packetHeaderSize = 2 * sizeof(int); // we have two "ints" for our header
	
	if(length < (1024 - packetHeaderSize)) { // our networkPacket buffer size minus the size of the header info
		int *pIntData = (int *)&networkPacket[0];
		// header info
		pIntData[0] = gamePacketNumber++;
		pIntData[1] = packetID;
		// copy data in after the header
		memcpy( &networkPacket[packetHeaderSize], data, length ); 
		
		NSData *packet = [NSData dataWithBytes: networkPacket length: (length+8)];
		if(howtosend == YES) { 
			[session sendData:packet toPeers:[NSArray arrayWithObject:gamePeerId] withDataMode:GKSendDataReliable error:nil];
		} else {
			[session sendData:packet toPeers:[NSArray arrayWithObject:gamePeerId] withDataMode:GKSendDataUnreliable error:nil];
		}
	}
}

- (void)receiveData:(NSData *)data fromPeer:(NSString *)peer inSession:(GKSession *)session context:(void *)context { 
	static int lastPacketTime = -1;
	unsigned char *incomingPacket = (unsigned char *)[data bytes];
	int *pIntData = (int *)&incomingPacket[0];

	int packetTime = pIntData[0];
	int packetID = pIntData[1];
	if(packetTime < lastPacketTime && packetID != NETWORK_COINTOSS) {
		return;	
	}
	
	lastPacketTime = packetTime;
	switch( packetID ) {
		case NETWORK_COINTOSS:
		{
			int coinToss = pIntData[2];
			if(coinToss > gameUniqueID) {
				self.peerStatus = kClient;
			}
			self.game_label.hidden = YES;
		}
		break;
		case NETWORK_GAME_STATUS:
		{
			gameInfo *gs = (gameInfo *)&incomingPacket[8];		
			memcpy( &gameStatus, gs, sizeof(gameInfo) );
			score_1_label.text = [NSString stringWithFormat:@"%d",gameStatus.score[kServer]];
			score_2_label.text = [NSString stringWithFormat:@"%d",gameStatus.score[kClient]];
		}
			break;
			
		case NETWORK_MOVE_EVENT:
		{
			gameInfo *gi = (gameInfo *)&incomingPacket[8];
			gameStatus.paddlePosition[1-self.peerStatus].x = gi->paddlePosition[1-self.peerStatus].x;
			
		}
			break;
	
		case NETWORK_BALL_MOVE_EVENT:
		{
			gameInfo * gi = (gameInfo *)&incomingPacket[8];
			gameStatus.ballPosition.x = gi->ballPosition.x;
			gameStatus.ballPosition.y = gi->ballPosition.y;
			gameStatus.ballVelocity.x = gi->ballVelocity.x;
			gameStatus.ballVelocity.y = gi->ballVelocity.y;
			break;
		}	
	}
}


- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

@end
