//
//  RootViewController.m
//  WPStore
//
//  Created by Brandon Trebitowski on 12/19/09.
//  Copyright RightSprite 2009. All rights reserved.
//

#import "RootViewController.h"
#import "WallpaperViewController.h"

@implementation RootViewController

@synthesize transactionHistory;

- (void)viewDidLoad {
    [super viewDidLoad];

	[[SKPaymentQueue defaultQueue] addTransactionObserver:self];

	products = [[NSMutableArray alloc] init];
	
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	NSString * path = [documentsDirectory stringByAppendingPathComponent:@"history.plist"];

	self.transactionHistory = [NSMutableArray arrayWithContentsOfFile:path];
	
	if(!transactionHistory) {
		NSMutableArray * _transactionHistory = [[NSMutableArray alloc] init];
		self.transactionHistory = _transactionHistory;
		[_transactionHistory release];
	}
	
	[self requestProductData];
}

- (void) requestProductData
{
	SKProductsRequest *request= [[SKProductsRequest alloc] initWithProductIdentifiers: 
								 [NSSet setWithObjects: @"com.rightsprite.wallpaper.01",
								  @"com.rightsprite.wallpaper.02",
								  @"com.rightsprite.wallpaper.03",nil]];
	request.delegate = self;
	[request start];
}
- (void)productsRequest:(SKProductsRequest *)request didReceiveResponse:(SKProductsResponse *)response
{
    NSArray *myProducts = response.products;
	for(SKProduct * product in myProducts) {
		[products addObject:product];
	}
	
    [request autorelease];
	[self.tableView reloadData];
}

- (void)paymentQueue:(SKPaymentQueue *)queue updatedTransactions:(NSArray *)transactions
{
    for (SKPaymentTransaction *transaction in transactions)
    {
        switch (transaction.transactionState)
        {
            case SKPaymentTransactionStatePurchased:
                [self completeTransaction:transaction];
                break;
            case SKPaymentTransactionStateFailed:
                [self failedTransaction:transaction];
                break;
            case SKPaymentTransactionStateRestored:
                [self restoreTransaction:transaction];
            default:
                break;
        }
    }
}

- (void) completeTransaction: (SKPaymentTransaction *)transaction
{
	[self.navigationController popViewControllerAnimated:YES];
    [self recordTransaction: transaction];
    [self provideContent: transaction.payment.productIdentifier];
    [[SKPaymentQueue defaultQueue] finishTransaction: transaction];
}

- (void) restoreTransaction: (SKPaymentTransaction *)transaction
{
	[self completeTransaction:transaction];
}

- (void) failedTransaction: (SKPaymentTransaction *)transaction
{
    if (transaction.error.code != SKErrorPaymentCancelled)
    {
        // Optionally, display an error here.
		UIAlertView * alert = [[UIAlertView alloc] initWithTitle:@"Error in purchase" message:transaction.error.description delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil];
		[alert show];
		[alert release];
    }
    [[SKPaymentQueue defaultQueue] finishTransaction: transaction];
}

- (void) recordTransaction:(SKPaymentTransaction * )transaction {
	
	if([self.transactionHistory containsObject:transaction.payment.productIdentifier]) return;
	
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	NSString * path = [documentsDirectory stringByAppendingPathComponent:@"history.plist"];
	[self.transactionHistory addObject:transaction.payment.productIdentifier];
	[self.transactionHistory writeToFile:path atomically:YES];
}

- (void) provideContent:(NSString * )productIdentifier {
	UIImageWriteToSavedPhotosAlbum([UIImage imageNamed:[NSString stringWithFormat:@"%@.jpeg",productIdentifier]],self, @selector(image:didFinishSavingWithError:contextInfo:),nil);
}

- (void)image:(UIImage *)image didFinishSavingWithError:(NSError *)error contextInfo:(void *)contextInfo {
	UIAlertView * alert = [[UIAlertView alloc] initWithTitle:@"Purchase Complete" message:@"The wallpaper has been saved to your camera roll." delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil];
	[alert show];
	[alert release];
}
								   
#pragma mark Table view methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [products count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier] autorelease];
    }
    
	SKProduct * product = [products objectAtIndex:indexPath.row];
	cell.textLabel.text = [NSString stringWithFormat:@"$%.2f %@",product.price.doubleValue, product.localizedTitle];
	cell.detailTextLabel.text = product.localizedDescription;
	cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
	
    return cell;
}

// Override to support row selection in the table view.
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	
	SKProduct * product = [products objectAtIndex:indexPath.row];
	
    WallpaperViewController * wpViewController = [[WallpaperViewController alloc] initWithNibName:@"WallpaperViewController" bundle:[NSBundle mainBundle]];
	wpViewController.product = product;
	[self.navigationController pushViewController:wpViewController animated:YES];
	[wpViewController release];
}



/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/


/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source.
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view.
    }   
}
*/


/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
}
*/


/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/


- (void)dealloc {
    [super dealloc];
	[products release];
	[transactionHistory release];
}


@end

